package org.colllib.introspect;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 *
 * @author mjackisch
 */
public class PropertyAccessor {
   private final String name;
   private Method readMethod;
   private Method writeMethod;
   private Field  publicField;

   public PropertyAccessor(String name) {
      this.name = name;
   }

   public String getName() {
      return name;
   }

   public Method getReadMethod() {
      return readMethod;
   }

   public void setReadMethod(Method readMethod) {
      this.readMethod = readMethod;
   }

   public Method getWriteMethod() {
      return writeMethod;
   }

   public void setWriteMethod(Method writeMethod) {
      this.writeMethod = writeMethod;
   }

   public Field getPublicField() {
      return publicField;
   }

   public void setPublicField(Field publicField) {
      this.publicField = publicField;
   }

   public boolean isWritable() {
      return writeMethod != null || publicField != null;
   }

   public Class<?> getPropertyType() {
      if(readMethod != null)
         return readMethod.getReturnType();
      else
         return publicField.getType();
   }

   public Object getPropertyValue(Object invokee) {
      try {
         if(readMethod != null)
            return readMethod.invoke(invokee, new Object[0]);
         return publicField.get(invokee);
      }
      catch(IllegalArgumentException e) {
         throw new RuntimeException(e);
      }
      catch(IllegalAccessException e) {
         throw new RuntimeException(e);
      }
      catch(InvocationTargetException e) {
         throw new RuntimeException(e);
      }
   }

   public void setPropertyValue(Object invokee, Object value) {
      if(writeMethod != null)
         try {
            writeMethod.invoke(invokee, new Object[] { value });
         }
         catch(IllegalArgumentException e) {
            throw new RuntimeException(e);
         }
         catch(IllegalAccessException e) {
            throw new RuntimeException(e);
         }
         catch(InvocationTargetException e) {
            throw new RuntimeException(e);
         }
      else if(publicField != null)
         try {
            publicField.set(invokee, value);
         }
         catch(IllegalArgumentException e) {
            throw new RuntimeException(e);
         }
         catch(IllegalAccessException e) {
            throw new RuntimeException(e);
         }
      else
         throw new UnsupportedOperationException("Property not writable " + name);
   }

   @Override
   public String toString() {
      return "PropertyAccessor [name=" + name + "]";
   }
}

