package org.colllib.util;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import org.colllib.filter.FilterCollection;

/**
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 * 
 * @author marc.jackisch
 */
public class StringUtil {
   private StringUtil() {}
   
   
   public static String emptyStringIfNull(String s) {
      return s == null ? "" : s;
   }
   
   public static String nullIfEmpty(String s) {
      if(s == null || s.length() == 0)
         return null;
      return s;
   }
   
   public static String defaultIfNull(String s, String def) {
      return s != null ? s : def;
   }
   
   public static String defaultIfNullOrEmpty(String s, String def) {
      return defaultIfNull(nullIfEmpty(s), def);
   }

   public static <T> String asString(Iterable<T> coll, String delim) {
      if(coll == null) 
         return "";
      StringBuilder sb = new StringBuilder();
      Iterator<T> i = coll.iterator();
      while(i.hasNext()) {
         sb.append(i.next().toString());
         if(i.hasNext()) sb.append(delim);
      }
      return sb.toString();
   }

   public static List<String> toCollection(String s, String d) {
      return s == null ? Collections.<String>emptyList() : CollectionUtil.asList(s.split(Pattern.quote(d)));
   }
   
   public static Iterable<String> nonEmptyIterable(String... strings) {
      return CollectionUtil.applyFilter(strings, FilterCollection.not(FilterCollection.<String>matches("")));
   }
}

