package org.colllib.util;

import java.util.List;


/**
 * TypeUtil: utility methods to work with java type conversions and generics.
 * Keeps those nasty unchecked warnings in one place
 * 
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 * 
 * @author marc.jackisch
 */
public class TypeUtil {
   private TypeUtil() {}
   
   /**
    * Type conversion / check of a list. Pass an untyped or wrongly typed list,
    * the elements target class, and get a correctly typed list or a class cast
    * exception.
    * 
    * @param in       the original list
    * @param tClass   the desired element class
    * @return a cast transformation of 
    */
   @SuppressWarnings("unchecked")
   public static <T> List<T> typecheck(List<?> in, Class<T> tClass) {
      if(in != null)
         for(Object t : in)
            tClass.cast(t); // throws class cast exception for wrong types
      return (List<T>) in;
   }
}
